#
# Copyright (C) 2002 Jp Calderone.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import interfaces, error

from twisted.python import components

class SchedulerAdapter(components.Adapter):
    __implements__ = interfaces.IScheduler,

    DELAYS = {
        interfaces.IScheduler.MINUTELY: 60,
        interfaces.IScheduler.HOURLY: 60 * 60,
        interfaces.IScheduler.DAILY: 60 * 60 * 24,
        interfaces.IScheduler.WEEKLY: 60 * 60 * 24 * 7,
        interfaces.IScheduler.MONTHLY: 60 * 60 * 24 * 30
    }

    def schedule(self, period, fn, *args, **kw):
        from twisted.internet import reactor
        def cycle():
            fn(*args, **kw)
            reactor.callLater(self.DELAYS[period], cycle)
        reactor.callLater(self.DELAYS[period], cycle)

    def __getstate__(self):
        d = self.__dict__.copy()
        d['scheduled'] = []
        return d

class StorageAdapter(components.Adapter):
    __implements__ = interfaces.IStorage,

    def __init__(self, original):
        components.Adapter.__init__(self, original)
        self.storage = {}

    def store(self, key, version, value):
        self.storage.setdefault(key, {})[version] = value

    def retrieve(self, key, version):
        if key in self.storage:
            if version in self.storage[key]:
                return self.storage[key][version]
            raise error.StorageVersionError, (key, version)
        raise error.StorageKeyError, (key, version)
