#
#   dict - A word spelling and lookup plugin for Twisted Pynfo
#   Copyright (C) 2003 Pavel Pergamenshchik (pp64@cornell.edu)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

# Twisted imports
from twisted.protocols import dict
#from twisted.python import log

HOST, PORT = 'dict.org', 2628
DATABASE, STRATEGY = 'wn', '.'

def info_DICT(bot, user, channel, query):
    """Usage: dict <word> [<database>] - /msg defition text to the user
<database> is an optional parameter specifying the word database. Try wn,
foldoc or gcide, or * to look in all databases."""
    def reply(result):
        if len(result) == 0:
            dict.match(HOST, PORT, sp[1], STRATEGY, sp[0]).addCallback(replyMatch).addErrback(oops)
            return
#        if len(result) == 1:
#            df = "definition"
#        else:
#            df = "definitions"
#        bot.reply(user, "%s: %d %s found" % (query, len(result), df))
#        for i in result:
        # grab only the first result
        i = result[0]
        bot.reply(user, "From %s [%s]:" % (i.dbdesc.encode("unicode-escape"), i.db.encode("unicode-escape")))
        for s in i.text:
            bot.reply(user, s.encode("unicode-escape"))

    def replyMatch(result):
#        log.write("info_DICT: replyMatch result is %s" % (str(result),))
        if len(result) == 0:
            bot.reply(user, "%s in %s: No definitions found" % (sp[0],sp[1]))
            return
        if len(result) == 1:
            m = "match"
        else:
            m = "matches"
        bot.reply(user, "%s in %s: No definitions found, however there are %d %s" % (sp[0], sp[1], len(result), m))
        res = ""
        for i in result:
            res += i[1] + ' '
        bot.reply(user, res.encode("unicode-escape"))

    def oops(reason):
        reason.printTraceback()
        bot.reply(user, "%s in %s: error occured" % (sp[0],sp[1]))

    sp = query.split(None, 1)
    if len(sp) == 0:
        bot.reply(channel, "dict: Not enough arguments")
        return
    if len(sp) == 1:
        sp.append(DATABASE)
    dict.define(HOST, PORT, sp[1], sp[0]).addCallback(reply).addErrback(oops)

def info_SPELL(bot, user, channel, query):
    """Usage: spell <word> [<database>] - offer corrections for misspelled <word>.
Match is performed using <database>."""
    def reply(result):
#        log.write("info_SPELL: reply result is %s" % (str(result),))
        if len(result) == 0:
            bot.reply(channel, "%s in %s: No matches found" % (sp[0], sp[1]))
            return
        resdict = {}
        for v,k in result:
            if k.lower() == sp[0].lower():
                res = "Exact match: " + k
                bot.reply(channel, res.encode("unicode-escape"))
                return
            resdict.setdefault(k, []).append(v)

#        log.write("info_SPELL: resdict is %s" % (str(resdict),))
#        res = ""
#        for i in resdict.keys():
#            res += i + ' (' + ' '.join(resdict[i]) + '); '
        w = resdict.keys()
        if len(w) == 1:
            m = "match"
        else:
            m = "matches"
        res = "%d %s: " % (len(w), m) + ' '.join(w)
        bot.reply(channel, res.encode("unicode-escape"))

    def oops(reason):
        reason.printTraceback()
        bot.reply(channel, "%s: error occured" % (sp[0], sp[1]))

    sp = query.split(None, 1)
    if len(sp) == 0:
        bot.reply(channel, "spell: Not enough arguments")
        return
    if len(sp) == 1:
        sp.append(DATABASE)

    dict.match(HOST, PORT, sp[1], STRATEGY, sp[0]).addCallback(reply).addErrback(oops)

