#
#   Twisted Pynfo - A relay, info, and search bot for IRC.
#   Copyright (C) 2002 Jp Calderone (exarkun@twistedmatrix.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

from twisted.protocols import http
from twisted.internet import protocol, reactor, threads
from twisted.web import client

import xml.sax      # GRRRRRRRR
import os

for p in ('/', '/usr/', '/usr/local/'):
    if os.path.exists(os.path.join(p, 'bin', 'bzip2')):
        BZIP2 = '/bin/bzip2'
        break
else:
    raise ImportError, "bzip2 not found"

class Bunzip2Protocol(protocol.ProcessProtocol):
    def __init__(self, callback):
        self.callback = callback
    
    def processEnded(self, arg):
        self.callback()


class FreshmeatProjectHandler(xml.sax.ContentHandler):
    mode = ''
    
    def startElement(self, name, attr):
        if name == 'project':
            self.projects.append({})
        elif name == 'projectname_full':
            self.mode = 'name'
        elif name == 'desc_full':
            self.mode = 'desc'
        elif name == 'desc_short':
            self.mode = 'short'
        elif name == 'url_homepage':
            self.mode = 'url'
        elif name == 'project-listing':
            self.projects = []


    def endElement(self, name):
        self.mode = ''
    

    def characters(self, chars):
        if self.mode in ('name', 'desc', 'short', 'url'):
            self.projects[-1][self.mode] = chars


class FreshmeatSearch:
    def __init__(self, projects, terms):
        self.projects = projects
        self.terms = terms.lower().split()
    
    def search(self):
        hits = []
        for i in self.projects:
            score = 0
            mult = len(self.terms)
            name, desc, short = i['name'].lower(), i['desc'].lower(), i['short'].lower()
            for t in self.terms:
                score += name.count(t) * 3 * mult
                score += short.count(t) * 2 * mult
                score += desc.count(t) * 1 * mult
                mult -= 1
            if score:
                hits.append((score, i))
        if hits:
            hits.sort()
            hit = hits[-1][1]
            return hit['name'], hit['short'], hit['url']
        return None

projects = None

def initiateSearch(terms, callback):
    reactor.callInThread(conductSearch, terms, callback)

def conductSearch(terms, callback):
    if conductSearch.projects is None:
        h = FreshmeatProjectHandler()
        p = xml.sax.make_parser()
        p.setContentHandler(h)
        p.parse(file('projects.rdf', 'r'))
        conductSearch.projects = []
        for project in h.projects:
            if len(project.keys()) == 4:
                conductSearch.projects.append(project)
    threads.deferToThread(FreshmeatSearch(conductSearch.projects, terms).search).addCallback(callback)
conductSearch.projects = None


def gotRDF(result, bot, channel, args):
    def tellChannel(answer):
        if answer:
            bot.reply(channel, ', '.join([s.encode('utf-8') for s in answer]))
        else:
            bot.reply(channel, "No matching projects found")
    
    proto = Bunzip2Protocol(lambda: initiateSearch(args, tellChannel))
    from twisted.internet import reactor
    reactor.spawnProcess(proto, BZIP2, ('bunzip2', 'projects.rdf.bz2'))

def searchComplete(answer, bot, channel):
    if answer:
        bot.reply(channel, ', '.join([s.encode('utf-8') for s in answer]))
    else:
        bot.reply(channel, "No matching projects found")

RDF_URL = 'http://download.freshmeat.net/backend/fm-projects.rdf.bz2'
def info_FM(bot, user, channel, args):
    """Usage: fm <search query> - Return the best match for the given query in the Freshmeat projects database"""
    if len(args):
        if not os.path.exists('projects.rdf'):
            client.downloadPage(RDF_URL, 'projects.rdf.bz2').addCallback(gotRDF, bot, channel, args)
            bot.reply(channel, 'Please wait - downloading projects.rdf')
        else:
            bot.reply(channel, 'Searching...')
            initiateSearch(args, lambda answer: searchComplete(answer, bot, channel))
