#
#   Twisted Pynfo - A relay, info, and search bot for IRC.
#   Copyright (C) 2002 Jp Calderone (exarkun@twistedmatrix.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

# Twisted imports
from twisted.protocols import http
from twisted.internet import protocol, reactor, defer

# System imports
import urllib

MODE = "I'm Feeling Lucky"
HOST, PORT = 'www.google.com', 80

class GoogleSearch(http.HTTPClient):
    def __init__(self, query, deferred):
        self.deferred = deferred
        self.query = query

    def connectionMade(self):
        self.sendCommand('GET', '/search?q=%s&btnI=%s' % (urllib.quote(self.query), urllib.quote(MODE)))
        self.sendHeader('Host', '%s:%d' % (HOST, PORT))
        self.sendHeader('User-agent', 'TwistZilla')
        self.endHeaders()

    def handleStatus(self, *args, **kwargs):
        if args[1] != '302':
            d, self.deferred = self.deferred, None
            d.errback(Exception("Unexpected Response"))


    def handleResponse(self, *args, **kwargs):
        pass


    def handleHeader(self, *args, **kwargs):
        if args[0] == 'Location' and self.deferred:
            d, self.deferred = self.deferred, None
            d.callback(args[1])


    def handleEndHeaders(self, *args, **kwargs):
        if self.deferred:
            d, self.deferred = self.deferred, None
            d.errback(Exception("Unexpected end of Response"))
        self.transport.loseConnection()


class GoogleFactory(protocol.ClientFactory):
    protocol = GoogleSearch

    def __init__(self, deferred, query):
        self.deferred = deferred
        self.query = query


    def clientConnectionLost(self, connector, reason):
        try:
            self.deferred.errback(reason)
        except defer.AlreadyCalledError:
            pass


    def clientConnectionFailed(self, connector, reason):
        try:
            self.deferred.errback(reason)
        except defer.AlreadyCalledError:
            pass


    def buildProtocol(self, addr):
        p = GoogleSearch(self.query, self.deferred)
        p.factory = self
        return p


def initiateSearch(terms):
    d = defer.Deferred()
    reactor.connectTCP(HOST, PORT, GoogleFactory(d, terms))
    return d


def info_GOOGLE(bot, user, channel, query):
    """Usage: google <search query> - Return the URL of the first google search result for the given query"""
    def reply(result):
        bot.reply(channel, "%s: %s" % (query, result))
    def oops(failure):
        failure.printTraceback()
        bot.reply(channel, "%s: No results found" % (query,))

    initiateSearch(query).addCallback(reply).addErrback(oops)
