#
# Copyright (C) 2002 Jp Calderone.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# System imports
import os

# Twisted imports
from twisted.python import reflect

# Pynfo imports
from Pynfo import info


def info_COMMANDS(bot, user, channel, args):
    """Usage: commands - List available commands.  -/+/* prefixes indicate disabled, privileged, and secure commands respectively."""
    cmds = reflect.prefixedMethodNames(bot.__class__, 'info_')
    pcmds = reflect.prefixedMethodNames(bot.__class__, 'privinfo_')
    scmds = reflect.prefixedMethodNames(bot.__class__, 'secure_')

    cmds.extend(bot.plugins['info_'].keys())
    pcmds.extend(bot.plugins['privinfo_'].keys())
    scmds.extend(bot.plugins['secure_'].keys())

    map(list.sort, (cmds, pcmds, scmds))
    cmds = [((x.lower() in bot.network.disabled and '-' or '') + x) for x in cmds] 
    pcmds = [((x.lower() in bot.network.disabled and '-+' or '+') + x) for x in pcmds]
    scmds = [((x.lower() in bot.network.disabled and '-*' or '*') + x) for x in scmds]
    bot.reply(channel, "Commands: %s" % (', '.join(cmds + pcmds + scmds).lower(),))


def info_PLUGINS(bot, user, channel, args):
    """Usage: plugins - List all loaded plugin modules."""
    f = lambda f: os.path.splitext(os.path.basename(f))[0]
    bot.reply(
        channel, 
        "Plugins: %s" % (
            ', '.join(
                [f(m.__file__) for m in bot.factory.pluginModules]
            ),
        )
    )

def info_SOURCE(bot, user, channel, args):
    """Usage: source - Reveal the location of my source code"""
    bot.reply(channel, "My source is available at " + bot.SRC_URL)


def info_HELP(bot, user, channel, args):
    """Usage: help <command> - Give help for <command>"""
    if len(args.strip()) == 0:
        raise info.UsageError("Get help on what?")

    args = args.upper()
    cmd = getattr(
        bot, 'info_' + args,
        getattr(
            bot, 'privinfo_' + args,
            getattr(bot, 'secure_' + args,
                bot.plugins['info_'].get(args,
                    bot.plugins['privinfo_'].get(args,
                        bot.plugins['secure_'].get(args, None)
                    )
                )
            )
        )
    )
    if cmd is None:
        raise info.UsageError("There is no such command.")
    elif cmd.__doc__ is None:
        raise info.UsageError("There is no help for that command.")
    else:
        bot.reply(channel, cmd.__doc__)
