#
#   Twisted Pynfo - A relay, info, and search bot for IRC.
#   Copyright (C) 2002 Jp Calderone (exarkun@twistedmatrix.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

__version__ = '$Revision: 1.6 $'[11:-2]

import os
import time
import xml.sax

from Pynfo import interfaces, error

from twisted.protocols import irc
from twisted.web import client
from twisted.python import components
from twisted.python import log

class Project:
    # Convenient place to store the project list
    currentProjects = None

    # Expected instance attributes
    title = link = description = pubDate = ''


class PyPiHandler(xml.sax.ContentHandler):
    mode = None
    projects = None

    def startElement(self, name, attr):
        if name == "channel":
            self.projects = []
        elif name == "item":
            self.projects.append(Project())
        elif name in ("title", "link", "description", "pubDate"):
            self.mode = name

    def endElement(self, name):
        self.mode = None

    def characters(self, chars):
        if self.mode and self.projects:
            setattr(self.projects[-1], self.mode, chars.encode('utf-8'))


def info_PYPI(bot, user, channel, args):
    """Usage: pypi - Display the project most recently registered with the Python Package Index"""
    if Project.currentProjects:
        p = Project.currentProjects[0]
        bot.reply(
            channel, 
            "Latest PyPI project: %s, %s, %s" % (
                p.title.strip(), p.description.strip(), p.link.strip()
            )
        )
    else:
        bot.reply(channel, "Nothing known about PyPI at the moment.")

notifyChannels = []
def privinfo_PKGINFO(bot, user, channel, args):
    """Usage: pkginfo <channel> - Add a channel to or remove a channel from the PyPI notification list"""
    args = args.strip().lower()
    if not args:
        if notifyChannels:
            bot.reply(channel, "Currently notifying " + ', '.join(notifyChannels))
        else:
            bot.reply(channel, "Currently notifying no one of PyPI updates.")
    elif args[0] not in irc.CHANNEL_PREFIXES and args != user:
        bot.reply(channel, "You can only add yourself to the notification list.")
    elif args in notifyChannels:
        notifyChannels.remove(args)
        bot.reply(channel, "Removed %s from PyPI notification list." % args)
    else:
        notifyChannels.append(args)
        bot.reply(channel, "Added %s to PyPI notification list." % args)

def examineDifferences(new, old, factory):
    if old:
        channels = []
        bots = factory.allBots.values()
        latest = old[0]
        for p in new:
            if p.pubDate.strip() <= latest.pubDate.strip():
                break
            else:
                for bot in bots:
                    for chan in notifyChannels:
                        if chan not in channels:
                            bot.reply(
                                chan, 'New PyPI posting: %s, %s, %s' % (
                                    p.title.strip(),
                                    p.description.strip(),
                                    p.link.strip()
                                )
                            )
                            channels.append(chan)
    return new


PYPI_RSS_URL="http://python.org/pypi?:action=rss"
def parseRSS():
    t = time.time()
    h = PyPiHandler()
    p = xml.sax.make_parser()
    p.setContentHandler(h)
    p.parse('pypi.rss')
    log.msg("Parsed PyPI RSS in %0.2f seconds" % (time.time() - t))
    return h.projects

def fetchLatest(factory):
    client.downloadPage(PYPI_RSS_URL, 'pypi.rss').addCallback(
        lambda result: parseRSS()
    ).addCallback(
        examineDifferences,
        Project.currentProjects, factory
    ).addCallback(
        lambda newProjects: setattr(Project, 'currentProjects', newProjects)
    )

def init(factory):
    sched = factory.getComponent(interfaces.IScheduler)
    sched.schedule(interfaces.IScheduler.HOURLY, fetchLatest, factory)

    global notifyChannels
    store = factory.getComponent(interfaces.IStorage)
    try:
        notifyChannels = store.retrieve('pypi', __version__)
    except error.RetrievalError:
        notifyChannels = []

    if not os.path.exists('pypi.rss'):
        fetchLatest(factory)
    else:
        Project.currentProjects = parseRSS()
        
def fini(factory):
    s = factory.getComponent(interfaces.IStorage)
    s.store('pypi', __version__, notifyChannels)
