#
#   Twisted Pynfo - A relay, info, and search bot for IRC.
#   Copyright (C) 2002 Jp Calderone (exarkun@twistedmatrix.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

import pymetar

# Twisted imports
from twisted.internet import defer
from twisted.web import client

class AsyncMetarReport(pymetar.MetarReport):
    def fetchMetarReport(self, station, baseurl="http://weather.noaa.gov/pub/data/observations/metar/decoded/"):
        """
        Retrieve the decoded METAR report from the given base URL and parse it.
        Fill the data properties in the class instance with the values found in
        the report, converting them to metric values where necessary.

        Return a deferred that will have its callback invoked with `self' when 
        the report has been fetched and processed.
        """
        self._ClearAllFields()
        station = station.upper()
        self.reporturl = "%s%s.TXT" % (baseurl, station)

        d = self.downloadReport(self.reporturl)
        d.addCallback(lambda r: self.processReport(station, r))
        d.addCallback(lambda r: self)
        return d

    def downloadReport(self, url):
        print 'Getting ', url
        return client.getPage(url)

def reportResult(report, bot, user, channel, query):
    s = """Current conditions at %s, %s: It is %d C (%d F), windspeed is %0.2f km/h, and visibility is %0.2f km. (Information current as of %s).""" % (
        report.getStationCity(), report.getStationCountry(),
        report.getTemperatureCelsius(), report.getTemperatureFahrenheit(),
        report.getWindSpeed() / 1000.0 * (60.0 * 60.0),
        report.getVisibilityKilometers(),
        report.getTime()
    )
    bot.reply(channel, s)

def reportError(failure, bot, user, channel, query):
    bot.reply(channel, "Failed to lookup weather at station %s" % query.upper())
    failure.printTraceback()

def weatherFailed(failure):
    failure.printTraceback()

def info_WEATHER(bot, user, channel, query):
    """Usage: weather <station code> - Report the weather at the given station (station codes can be looked up at http://www.nws.noaa.gov/tg/siteloc.shtml)"""
    AsyncMetarReport().fetchMetarReport(query).addCallbacks(
        reportResult,
        reportError,
        callbackArgs=(bot, user, channel, query),
        errbackArgs=(bot, user, channel, query)
    ).addErrback(weatherFailed)

